@@ -331,8 +331,13 @@ fr_command_7z_add (FrCommand     *comm,
 	fr_process_add_arg (comm->process, "-y");
 	fr_process_add_arg (comm->process, "-l");
 	add_password_arg (comm, comm->password, FALSE);
-	if ((comm->password != NULL) && (*comm->password != 0) && comm->encrypt_header)
+	if ((comm->password != NULL)
+	    && (*comm->password != 0)
+	    && comm->encrypt_header
+	    && fr_command_is_capable_of (comm, FR_COMMAND_CAN_ENCRYPT_HEADER))
+	{
 		fr_process_add_arg (comm->process, "-mhe=on");
+	}
 
 	/* fr_process_add_arg (comm->process, "-ms=off"); FIXME: solid mode off? */
 
@@ -512,7 +517,7 @@ fr_command_7z_handle_error (FrCommand   *comm,
 
 const char *sevenz_mime_types[] = { "application/x-7z-compressed",
 				    "application/x-arj",
-				    "application/x-cabinet",
+				    "application/vnd.ms-cab-compressed",
 				    "application/x-cd-image",
 				    /*"application/x-cbr",*/
 				    "application/x-cbz",
@@ -562,6 +567,16 @@ fr_command_7z_get_capabilities (FrCommand  *comm,
 			capabilities |= FR_COMMAND_CAN_WRITE | FR_COMMAND_CAN_ENCRYPT;
 		}
 	}
+	else if (is_program_in_path ("7za")) {
+		if (is_mime_type (mime_type, "application/vnd.ms-cab-compressed")
+		    || is_mime_type (mime_type, "application/zip"))
+		{
+			capabilities |= FR_COMMAND_CAN_READ;
+		}
+
+		if (is_mime_type (mime_type, "application/zip"))
+			capabilities |= FR_COMMAND_CAN_WRITE;
+	}
 
 	/* multi-volumes are read-only */
 	if ((comm->files->len > 0) && comm->multi_volume && (capabilities & FR_COMMAND_CAN_WRITE))
